var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { Auth, AxiosAdapter, DefaultTokenStorage, JwtTokenDecoder, OpenidConnectAdapter, TokenStorageAsyncAdapter } from '../src';
import axios from 'axios';
import MockAdapter from 'axios-mock-adapter';
describe('Auth', function () {
    beforeEach(function () {
        localStorage.clear();
        sessionStorage.clear();
    });
    afterEach(function () {
        localStorage.clear();
        sessionStorage.clear();
    });
    it('defines a default module', function () {
        expect(Auth).toBeDefined();
    });
    it('loads token from sessionStorage', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        var token = auth.getTokens();
        expect(token).toBeDefined();
        if (token != null) {
            expect(token.access.value).toBe('accessTokenValue');
            expect(token.refresh).toBeUndefined();
        }
        expect(localStorage.getItem('auth.accessToken')).toBe(null);
    });
    it('does not load token from sessionStorage when loadTokensFromStorage is false', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            loadTokensFromStorage: false
        });
        var token = auth.getTokens();
        expect(token).toBeUndefined();
    });
    it('loads tokens from localStorage', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        localStorage.setItem('auth.accessToken', 'accessTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        auth.loadTokensFromStorage();
        var token = auth.getTokens();
        expect(token).toBeDefined();
        if (token != null) {
            expect(token.access.value).toBe('accessTokenValue');
            expect(token.refresh).toBeUndefined();
        }
        expect(sessionStorage.getItem('auth.accessToken')).toBe('accessTokenValue');
    });
    it('loads tokens from sync localStorage (async)', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, openidConnectAdapter, serverConfiguration, auth, token;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    localStorage.setItem('auth.accessToken', 'accessTokenValue');
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    return [4 /*yield*/, auth.loadTokensFromStorageAsync()];
                case 1:
                    _a.sent();
                    token = auth.getTokens();
                    expect(token).toBeDefined();
                    if (token != null) {
                        expect(token.access.value).toBe('accessTokenValue');
                        expect(token.refresh).toBeUndefined();
                    }
                    expect(sessionStorage.getItem('auth.accessToken')).toBe('accessTokenValue');
                    return [2 /*return*/];
            }
        });
    }); });
    it('loads tokens from async localStorage (async)', function (done) {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        var tokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(sessionStorage), false);
        var persistentTokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(localStorage), false);
        localStorage.setItem('auth.accessToken', 'accessTokenValue');
        var listener = {
            initialized: function (loaded, err) {
                expect(loaded).toBeTruthy();
                expect(err).toBeUndefined();
                var token = auth.getTokens();
                expect(token).toBeDefined();
                if (token != null) {
                    expect(token.access.value).toBe('accessTokenValue');
                    expect(token.refresh).toBeUndefined();
                }
                expect(sessionStorage.getItem('auth.accessToken')).toBe('accessTokenValue');
                done();
            }
        };
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            tokenStorage: tokenStorage,
            persistentTokenStorage: persistentTokenStorage,
            loadTokensFromStorage: true,
            listeners: [listener]
        });
    });
    it('loads tokens from async localStorage (async)', function (done) {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        var FailingTokenStorage = /** @class */ (function () {
            function FailingTokenStorage() {
                this.async = false;
            }
            FailingTokenStorage.prototype.clear = function () {
                throw new Error('!FailingTokenStorage!');
            };
            FailingTokenStorage.prototype.getTokens = function () {
                throw new Error('!FailingTokenStorage!');
            };
            FailingTokenStorage.prototype.store = function (tokens) {
                throw new Error('!FailingTokenStorage!');
            };
            return FailingTokenStorage;
        }());
        var tokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(sessionStorage), false);
        var persistentTokenStorage = new TokenStorageAsyncAdapter(new FailingTokenStorage(), false);
        localStorage.setItem('auth.accessToken', 'accessTokenValue');
        var listener = {
            initialized: function (loaded, err) {
                expect(loaded).toBeFalsy();
                expect(err).toBeInstanceOf(Error);
                done();
            }
        };
        // eslint-disable-next-line no-new
        new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            tokenStorage: tokenStorage,
            persistentTokenStorage: persistentTokenStorage,
            loadTokensFromStorage: true,
            listeners: [listener]
        });
    });
    it('fails to load tokens from async localStorage (sync)', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        var tokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(sessionStorage), false);
        var persistentTokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(localStorage), false);
        localStorage.setItem('auth.accessToken', 'accessTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            tokenStorage: tokenStorage,
            persistentTokenStorage: persistentTokenStorage
        });
        expect(function () { return auth.loadTokensFromStorage(); }).toThrow('tokenStorage is async. Use loadTokensFromStorageAsync method instead');
    });
    it('loads tokens from storage with undefined TokenStorage', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        localStorage.setItem('auth.accessToken', 'accessTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            accessTokenDecoder: null,
            tokenStorage: null,
            persistentTokenStorage: null
        });
        auth.loadTokensFromStorage();
    });
    it('loads tokens from storage with defined SessionStorageAdapter', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        localStorage.setItem('auth.accessToken', 'accessTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            persistentTokenStorage: null
        });
        auth.loadTokensFromStorage();
        var token = auth.getTokens();
        expect(token).toBeUndefined();
        expect(sessionStorage.getItem('auth.accessToken')).toBeNull();
    });
    it('updates localStorage from sessionStorage when loading tokens from storage', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
        localStorage.setItem('auth.accessToken', 'accessTokenValueDummy');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        auth.loadTokensFromStorage();
        var token = auth.getTokens();
        expect(token).toEqual({ access: { value: 'accessTokenValue' } });
        expect(sessionStorage.getItem('auth.accessToken')).toBe('accessTokenValue');
        expect(localStorage.getItem('auth.accessToken')).toBe('accessTokenValue');
    });
    it('loads tokens from storage when localStorage is empty', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        auth.loadTokensFromStorage();
        var token = auth.getTokens();
        expect(token).toEqual({ access: { value: 'accessTokenValue' } });
        expect(sessionStorage.getItem('auth.accessToken')).toBe('accessTokenValue');
        expect(localStorage.getItem('auth.accessToken')).toBeNull();
    });
    it('loads tokens from storage when sessionStorage and localStorage are empty', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        auth.loadTokensFromStorage();
        var token = auth.getTokens();
        expect(token).toBeUndefined();
        expect(sessionStorage.getItem('auth.accessToken')).toBeNull();
        expect(localStorage.getItem('auth.accessToken')).toBeNull();
    });
    it('accessToken and refreshToken are defined after loading tokens from storage', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
        sessionStorage.setItem('auth.refreshToken', 'refreshTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        auth.loadTokensFromStorage();
        var tokens = auth.getTokens();
        expect(tokens).toBeDefined();
        if (tokens != null) {
            expect(tokens.access.value).toEqual('accessTokenValue');
            expect(tokens.refresh).toBeDefined();
            if (tokens.refresh != null) {
                expect(tokens.refresh.value).toEqual('refreshTokenValue');
            }
        }
    });
    it('decodes accessToken with defined tokenDecoder.decode', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var accessToken = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM3MSwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.uXiL5Yu-Ip0iNkvmK54U5MHDEhE0M6KsNFAb-BWg6oQ';
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', accessToken);
        var accessTokenDecoder = new JwtTokenDecoder();
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            accessTokenDecoder: accessTokenDecoder
        });
        auth.loadTokensFromStorage();
        var decodedToken = auth.decodeAccessToken();
        expect(decodedToken).toEqual({
            iss: 'Jwt Toolbox',
            iat: 133713311,
            exp: 133713371,
            aud: 'jwt-toolbox',
            sub: 'jwt-toolbox'
        });
    });
    it('check accessToken is expired defined tokenDecoder.isExpired', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var accessToken = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM3MSwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.uXiL5Yu-Ip0iNkvmK54U5MHDEhE0M6KsNFAb-BWg6oQ';
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', accessToken);
        var accessTokenDecoder = new JwtTokenDecoder();
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            accessTokenDecoder: accessTokenDecoder
        });
        auth.loadTokensFromStorage();
        var expiredToken = auth.isExpiredAccessToken();
        expect(expiredToken).toBe(true);
    });
    it('does not decode accessToken with undefined tokenDecoder.decode', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var accessToken = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM3MSwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.uXiL5Yu-Ip0iNkvmK54U5MHDEhE0M6KsNFAb-BWg6oQ';
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', accessToken);
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        auth.loadTokensFromStorage();
        expect(function () {
            auth.decodeAccessToken();
        }).toThrow('An accessTokenDecoder supporting decode method should be defined to decode access token.');
    });
    it('does not decode accessToken with null tokenDecoder', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var accessToken = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM3MSwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.uXiL5Yu-Ip0iNkvmK54U5MHDEhE0M6KsNFAb-BWg6oQ';
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', accessToken);
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            accessTokenDecoder: null
        });
        auth.loadTokensFromStorage();
        try {
            auth.decodeAccessToken();
            expect(false).toBeTruthy();
        }
        catch (e) {
            expect(function () {
                throw e;
            }).toThrow('');
        }
    });
    it('is authenticated after loading tokens from storage', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, openidConnectAdapter, serverConfiguration, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
                    sessionStorage.setItem('auth.refreshToken', 'refreshTokenValue');
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    auth.loadTokensFromStorage();
                    expect(auth.isAuthenticated()).toBeTruthy();
                    return [4 /*yield*/, auth.logout()];
                case 1:
                    _a.sent();
                    expect(auth.isAuthenticated()).toBeFalsy();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('isSaveCredentials is true when localStorage contains token values and loading tokens from storage', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        localStorage.setItem('auth.accessToken', 'accessTokenValue');
        localStorage.setItem('auth.refreshToken', 'refreshTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        auth.loadTokensFromStorage();
        expect(auth.usePersistentStorage).toBeTruthy();
    });
    it('isSaveCredentials is false when localStorage is empty and loading token from storage', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
        sessionStorage.setItem('auth.refreshToken', 'refreshTokenValue');
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
        auth.loadTokensFromStorage();
        expect(auth.usePersistentStorage).toBeFalsy();
    });
    it('logs in', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, listener, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    listener = {
                        initialized: jest.fn(),
                        login: jest.fn(),
                        renew: jest.fn(),
                        logout: jest.fn(),
                        expired: jest.fn(),
                        tokensChanged: jest.fn()
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        listeners: [listener]
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    expect(auth.isAuthenticated());
                    expect(listener.login).toHaveBeenCalledTimes(1);
                    expect(listener.login).toHaveBeenLastCalledWith();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(2);
                    expect(listener.tokensChanged).toHaveBeenNthCalledWith(1);
                    expect(listener.tokensChanged).toHaveBeenNthCalledWith(2, {
                        access: { value: 'accessTokenValue' },
                        refresh: { value: 'refreshTokenValue' }
                    });
                    auth.removeListener(listener);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 2:
                    _a.sent();
                    expect(listener.login).toHaveBeenCalledTimes(1);
                    expect(listener.login).toHaveBeenLastCalledWith();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(2);
                    expect(listener.tokensChanged).toHaveBeenNthCalledWith(1);
                    expect(listener.tokensChanged).toHaveBeenNthCalledWith(2, {
                        access: { value: 'accessTokenValue' },
                        refresh: { value: 'refreshTokenValue' }
                    });
                    expect(listener.renew).not.toHaveBeenCalled();
                    expect(listener.logout).not.toHaveBeenCalled();
                    expect(listener.expired).not.toHaveBeenCalled();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('logs in with save credentials defined and renewEndpoint defined', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, listener, tokens;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        renewEndpoint: { method: 'post', url: 'renew' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    listener = {
                        login: jest.fn(),
                        renew: jest.fn(),
                        logout: jest.fn(),
                        expired: jest.fn(),
                        tokensChanged: jest.fn()
                    };
                    auth.addListener(listener);
                    expect(auth.usePersistentStorage).toBeFalsy();
                    auth.usePersistentStorage = true;
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    expect(auth.usePersistentStorage).toBeTruthy();
                    tokens = auth.getTokens();
                    expect(tokens).toBeDefined();
                    if (tokens != null) {
                        expect(tokens.credentials).toBeUndefined();
                    }
                    expect(localStorage.getItem('auth.credentials')).toBeNull();
                    auth.usePersistentStorage = false;
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 2:
                    _a.sent();
                    expect(auth.usePersistentStorage).toBeFalsy();
                    tokens = auth.getTokens();
                    expect(tokens).toBeDefined();
                    if (tokens != null) {
                        expect(tokens.credentials).toBeUndefined();
                    }
                    expect(localStorage.getItem('auth.credentials')).toBeNull();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('logs in with save credentials defined and renewEndpoint undefined', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, listener, tokens;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    listener = {
                        login: jest.fn(),
                        renew: jest.fn(),
                        logout: jest.fn(),
                        expired: jest.fn(),
                        tokensChanged: jest.fn()
                    };
                    auth.addListener(listener);
                    auth.usePersistentStorage = true;
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    tokens = auth.getTokens();
                    expect(tokens).toBeDefined();
                    if (tokens != null) {
                        expect(tokens.credentials).toBeDefined();
                    }
                    expect(localStorage.getItem('auth.credentials')).not.toBeNull();
                    expect(JSON.parse(localStorage.getItem('auth.credentials'))).toEqual({
                        username: 'testUsername',
                        password: 'testPassword'
                    });
                    auth.usePersistentStorage = false;
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 2:
                    _a.sent();
                    tokens = auth.getTokens();
                    expect(tokens).toBeDefined();
                    if (tokens != null) {
                        expect(tokens.credentials).toBeUndefined();
                    }
                    expect(localStorage.getItem('auth.credentials')).toBeNull();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('logs out with undefined logoutEndpoint', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, listener;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    localStorage.setItem('auth.accessToken', 'accessTokenValue');
                    localStorage.setItem('auth.refreshToken', 'refreshTokenValue');
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    auth.loadTokensFromStorage();
                    expect(auth.isAuthenticated()).toBeTruthy();
                    listener = {
                        login: jest.fn(),
                        renew: jest.fn(),
                        logout: jest.fn(),
                        expired: jest.fn(),
                        tokensChanged: jest.fn()
                    };
                    auth.addListener(listener);
                    return [4 /*yield*/, auth.logout()];
                case 1:
                    _a.sent();
                    expect(auth.isAuthenticated()).toBeFalsy();
                    expect(listener.login).not.toHaveBeenCalled();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(1);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith();
                    expect(listener.renew).not.toHaveBeenCalled();
                    expect(listener.logout).toHaveBeenCalledTimes(1);
                    expect(listener.login).not.toHaveBeenLastCalledWith();
                    auth.removeListener(listener);
                    auth.usePersistentStorage = true;
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 2:
                    _a.sent();
                    return [4 /*yield*/, auth.logout()];
                case 3:
                    _a.sent();
                    expect(listener.login).not.toHaveBeenCalled();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(1);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith();
                    expect(listener.renew).not.toHaveBeenCalled();
                    expect(listener.logout).toHaveBeenCalledTimes(1);
                    expect(listener.login).not.toHaveBeenLastCalledWith();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('logs out with defined logoutEndpoint', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, listener;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('logout').reply(200);
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        logoutEndpoint: { method: 'post', url: 'logout' }
                    };
                    localStorage.setItem('auth.accessToken', 'accessTokenValue');
                    localStorage.setItem('auth.refreshToken', 'refreshTokenValue');
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    auth.loadTokensFromStorage();
                    expect(auth.isAuthenticated()).toBeTruthy();
                    listener = {
                        login: jest.fn(),
                        renew: jest.fn(),
                        logout: jest.fn(),
                        expired: jest.fn(),
                        tokensChanged: jest.fn()
                    };
                    auth.addListener(listener);
                    return [4 /*yield*/, auth.logout()];
                case 1:
                    _a.sent();
                    expect(auth.isAuthenticated()).toBeFalsy();
                    expect(listener.login).not.toHaveBeenCalled();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(1);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith();
                    expect(listener.renew).not.toHaveBeenCalled();
                    expect(listener.logout).toHaveBeenCalledTimes(1);
                    expect(listener.login).not.toHaveBeenLastCalledWith();
                    auth.removeListener(listener);
                    auth.usePersistentStorage = true;
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 2:
                    _a.sent();
                    return [4 /*yield*/, auth.logout()];
                case 3:
                    _a.sent();
                    expect(listener.login).not.toHaveBeenCalled();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(1);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith();
                    expect(listener.renew).not.toHaveBeenCalled();
                    expect(listener.logout).toHaveBeenCalledTimes(1);
                    expect(listener.login).not.toHaveBeenLastCalledWith();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('renews with defined renewEndpoint', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, listener;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        renewEndpoint: { method: 'post', url: 'renew' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    listener = {
                        login: jest.fn(),
                        renew: jest.fn(),
                        logout: jest.fn(),
                        expired: jest.fn(),
                        tokensChanged: jest.fn()
                    };
                    auth.addListener(listener);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    expect(auth.isAuthenticated());
                    expect(listener.login).toHaveBeenCalledTimes(1);
                    expect(listener.login).toHaveBeenLastCalledWith();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(1);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith({
                        access: { value: 'accessTokenValue' },
                        refresh: { value: 'refreshTokenValue' }
                    });
                    return [4 /*yield*/, auth.renew()];
                case 2:
                    _a.sent();
                    expect(listener.login).toHaveBeenCalledTimes(1);
                    expect(listener.login).toHaveBeenLastCalledWith();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(2);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith({
                        access: { value: 'accessTokenValueRenew' },
                        refresh: { value: 'refreshTokenValueRenew' }
                    });
                    expect(listener.renew).toHaveBeenCalledTimes(1);
                    expect(listener.logout).not.toHaveBeenCalled();
                    expect(listener.expired).not.toHaveBeenCalled();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('fails to renew with defined renewEndpoint and missing refresh_token in response', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        renewEndpoint: { method: 'post', url: 'renew' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    expect(auth.isAuthenticated());
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, auth.renew()];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_1 = _a.sent();
                    expect(function () {
                        throw e_1;
                    }).toThrow('Can\'t renew with the current state.');
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('renews only once when multiple requests are performed', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, listener;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        renewEndpoint: { method: 'post', url: 'renew' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    listener = {
                        login: jest.fn(),
                        renew: jest.fn(),
                        logout: jest.fn(),
                        expired: jest.fn(),
                        tokensChanged: jest.fn()
                    };
                    auth.addListener(listener);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    expect(auth.isAuthenticated());
                    expect(listener.login).toHaveBeenCalledTimes(1);
                    expect(listener.login).toHaveBeenLastCalledWith();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(1);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith({
                        access: { value: 'accessTokenValue' },
                        refresh: { value: 'refreshTokenValue' }
                    });
                    return [4 /*yield*/, Promise.all([auth.renew(), auth.renew(), auth.renew()])];
                case 2:
                    _a.sent();
                    expect(listener.login).toHaveBeenCalledTimes(1);
                    expect(listener.login).toHaveBeenLastCalledWith();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(2);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith({
                        access: { value: 'accessTokenValueRenew' },
                        refresh: { value: 'refreshTokenValueRenew' }
                    });
                    expect(listener.renew).toHaveBeenCalledTimes(1);
                    expect(listener.logout).not.toHaveBeenCalled();
                    expect(listener.expired).not.toHaveBeenCalled();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('fails to renew only once when multiple requests are performed', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, listener, e_2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('renew').reply(400, {
                        error: 'invalid_grant'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        renewEndpoint: { method: 'post', url: 'renew' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    listener = {
                        login: jest.fn(),
                        renew: jest.fn(),
                        logout: jest.fn(),
                        expired: jest.fn(),
                        tokensChanged: jest.fn()
                    };
                    auth.addListener(listener);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    expect(auth.isAuthenticated());
                    expect(listener.login).toHaveBeenCalledTimes(1);
                    expect(listener.login).toHaveBeenLastCalledWith();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(1);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith({
                        access: { value: 'accessTokenValue' },
                        refresh: { value: 'refreshTokenValue' }
                    });
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, Promise.all([auth.renew(), auth.renew(), auth.renew()])];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_2 = _a.sent();
                    expect(function () {
                        throw e_2;
                    }).toThrow('');
                    return [3 /*break*/, 5];
                case 5:
                    expect(listener.login).toHaveBeenCalledTimes(1);
                    expect(listener.login).toHaveBeenLastCalledWith();
                    expect(listener.tokensChanged).toHaveBeenCalledTimes(2);
                    expect(listener.tokensChanged).toHaveBeenLastCalledWith();
                    expect(listener.logout).not.toHaveBeenCalled();
                    expect(listener.expired).toHaveBeenCalledTimes(1);
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('intercepts with undefined token decoder and expired access token', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, accessTokenDecoder, auth, e_3;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    accessTokenDecoder = {
                        isExpired: function (token) {
                            return token.value === 'accessTokenValue';
                        }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: accessTokenDecoder
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    return [3 /*break*/, 5];
                case 4:
                    e_3 = _a.sent();
                    expect(function () {
                        throw e_3;
                    }).not.toThrow();
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('intercepts with expired renew token', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, accessTokenDecoder, auth, e_4;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(400, {
                        error: 'invalid_grant'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    accessTokenDecoder = {
                        isExpired: function (token) {
                            return token.value === 'accessTokenValue';
                        }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: accessTokenDecoder
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_4 = _a.sent();
                    expect(function () {
                        throw e_4;
                    }).toThrow(/Request failed with status code 400.*/);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('intercepts with null token decoder and expired access token', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, accessTokenDecoder, auth, e_5;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    accessTokenDecoder = null;
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: accessTokenDecoder
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    return [3 /*break*/, 5];
                case 4:
                    e_5 = _a.sent();
                    expect(function () {
                        throw e_5;
                    }).not.toThrow();
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('intercepts with null token decoder and expired access token and expired refresh token', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, tokenDecoder, auth, e_6;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(401, {
                        error: 'invalid_grant'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    tokenDecoder = null;
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, tokenDecoder);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_6 = _a.sent();
                    expect(function () {
                        throw e_6;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('intercepts with null token decoder and expired access token and no refresh token', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, tokenDecoder, auth, e_7;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(401, {
                        error: 'invalid_grant'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' }
                    };
                    tokenDecoder = null;
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, tokenDecoder);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_7 = _a.sent();
                    expect(function () {
                        throw e_7;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('intercepts with token decoder and expired access token and no refresh token', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, ExpiredTokenDecoder, accessTokenDecoder, auth, e_8;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(401, {
                        error: 'invalid_grant'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' }
                    };
                    ExpiredTokenDecoder = /** @class */ (function () {
                        function ExpiredTokenDecoder() {
                        }
                        ExpiredTokenDecoder.prototype.isExpired = function (token) {
                            return true;
                        };
                        return ExpiredTokenDecoder;
                    }());
                    accessTokenDecoder = new ExpiredTokenDecoder();
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, { accessTokenDecoder: accessTokenDecoder });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_8 = _a.sent();
                    expect(function () {
                        throw e_8;
                    }).toThrow('Access token is expired');
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('does not intercept after release', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, tokenDecoder, auth, e_9;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    tokenDecoder = null;
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, tokenDecoder);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    auth.release();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_9 = _a.sent();
                    expect(function () {
                        throw e_9;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('does not intercept if clientInterceptors is false', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, e_10;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        clientInterceptors: false
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_10 = _a.sent();
                    expect(function () {
                        throw e_10;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('does not intercept if matching a string exclude', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, e_11;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        excludes: [/.*usto.*/]
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_11 = _a.sent();
                    expect(function () {
                        throw e_11;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('does not intercept if matching a RegExp exclude', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, exclude, auth, e_12;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    exclude = function (request, response) {
                        return request.url === 'custom';
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        excludes: [exclude]
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_12 = _a.sent();
                    expect(function () {
                        throw e_12;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('does not intercept if matching a functional exclude', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, e_13;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValueRenew') {
                            return [200];
                        }
                        else {
                            return [
                                401,
                                {
                                    error: 'invalid_token'
                                }
                            ];
                        }
                    });
                    axiosMock.onPost('login').reply(function (config) {
                        if (config.data === 'grant_type=password&username=testUsername&password=testPassword') {
                            return [
                                200,
                                {
                                    access_token: 'accessTokenValue',
                                    refresh_token: 'refreshTokenValue'
                                }
                            ];
                        }
                        else {
                            return [401];
                        }
                    });
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'POST', url: 'login' },
                        renewEndpoint: { method: 'POST', url: 'renew' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        excludes: ['custom']
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_13 = _a.sent();
                    expect(function () {
                        throw e_13;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, null];
            }
        });
    }); });
    it('logs in, renew and logs out with undefined TokenStorage and undefined renewEndpoint', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, e_14;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('logout').reply(200);
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        logoutEndpoint: { method: 'post', url: 'logout' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: null,
                        tokenStorage: null,
                        persistentTokenStorage: null
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, auth.renew()];
                case 3:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 5];
                case 4:
                    e_14 = _a.sent();
                    expect(function () {
                        throw e_14;
                    }).toThrow('Can\'t renew with the current state.');
                    return [3 /*break*/, 5];
                case 5: return [4 /*yield*/, auth.logout()];
                case 6:
                    _a.sent();
                    auth.usePersistentStorage = true;
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 7:
                    _a.sent();
                    return [4 /*yield*/, auth.renew()];
                case 8:
                    _a.sent();
                    return [4 /*yield*/, auth.logout()];
                case 9:
                    _a.sent();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('logs in, renew and logs out with undefined TokenStorage and defined renewEndpoint', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('logout').reply(200);
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        renewEndpoint: { method: 'post', url: 'renew' },
                        logoutEndpoint: { method: 'post', url: 'logout' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: null,
                        tokenStorage: null,
                        persistentTokenStorage: null
                    });
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, auth.renew()];
                case 2:
                    _a.sent();
                    return [4 /*yield*/, auth.logout()];
                case 3:
                    _a.sent();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('logs in, renew and logs out with async server configuration', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, asyncServerConfiguration, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('logout').reply(200);
                    axiosMock.onPost('renew').reply(200, {
                        access_token: 'accessTokenValueRenew',
                        refresh_token: 'refreshTokenValueRenew'
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    asyncServerConfiguration = new Promise(function (resolve) {
                        var serverConfiguration = {
                            loginEndpoint: { method: 'post', url: 'login' },
                            renewEndpoint: { method: 'post', url: 'renew' },
                            logoutEndpoint: { method: 'post', url: 'logout' }
                        };
                        setTimeout(function () {
                            resolve(serverConfiguration);
                        }, 100);
                    });
                    auth = new Auth(asyncServerConfiguration, openidConnectAdapter, axiosAdapter);
                    return [4 /*yield*/, auth.login({ username: 'testUsername', password: 'testPassword' })];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, auth.renew()];
                case 2:
                    _a.sent();
                    return [4 /*yield*/, auth.logout()];
                case 3:
                    _a.sent();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('calls tokensChanged callback after loading of tokenStorage', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        /*
        const axiosMock: MockAdapter = new MockAdapter(axiosInstance)
        */
        var openidConnectAdapter = new OpenidConnectAdapter();
        var asyncServerConfiguration = new Promise(function (resolve) {
            var serverConfiguration = {
                loginEndpoint: { method: 'post', url: 'login' },
                renewEndpoint: { method: 'post', url: 'renew' },
                logoutEndpoint: { method: 'post', url: 'logout' }
            };
            setTimeout(function () {
                resolve(serverConfiguration);
            }, 100);
        });
        sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
        var auth = new Auth(asyncServerConfiguration, openidConnectAdapter, axiosAdapter);
        var listener = {
            tokensChanged: jest.fn()
        };
        auth.addListener(listener);
        auth.loadTokensFromStorage();
        expect(listener.tokensChanged).toHaveBeenCalledTimes(1);
        expect(auth.getTokens()).toEqual({ access: { value: 'accessTokenValue' } });
    });
    it('logs out with undefined tokens', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('logout').reply(200);
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        logoutEndpoint: { method: 'post', url: 'logout' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: null,
                        tokenStorage: null,
                        persistentTokenStorage: null
                    });
                    return [4 /*yield*/, auth.logout()];
                case 1:
                    _a.sent();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('renews with undefined tokens', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onPost('login').reply(200, {
                        access_token: 'accessTokenValue',
                        refresh_token: 'refreshTokenValue'
                    });
                    axiosMock.onPost('logout').reply(200);
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' },
                        logoutEndpoint: { method: 'post', url: 'logout' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: null,
                        tokenStorage: null,
                        persistentTokenStorage: null
                    });
                    return [4 /*yield*/, auth.renew()];
                case 1:
                    _a.sent();
                    return [2 /*return*/, null];
            }
        });
    }); });
    it('sends Bearer when accessToken is manually defined (async)', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, e_15, e_16;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValue') {
                            return [200];
                        }
                        else {
                            return [401];
                        }
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    return [4 /*yield*/, auth.setTokensAsync({ access: { value: 'accessTokenValue' } })];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 3:
                    _a.sent();
                    return [3 /*break*/, 5];
                case 4:
                    e_15 = _a.sent();
                    expect(function () {
                        throw e_15;
                    }).not.toThrow();
                    return [3 /*break*/, 5];
                case 5: return [4 /*yield*/, auth.setTokensAsync(null)];
                case 6:
                    _a.sent();
                    _a.label = 7;
                case 7:
                    _a.trys.push([7, 9, , 10]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 8:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 10];
                case 9:
                    e_16 = _a.sent();
                    expect(function () {
                        throw e_16;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 10];
                case 10: return [2 /*return*/, null];
            }
        });
    }); });
    it('sends Bearer when accessToken is manually defined (sync)', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth, e_17, e_18;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValue') {
                            return [200];
                        }
                        else {
                            return [401];
                        }
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    auth.setTokens({ access: { value: 'accessTokenValue' } });
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 2:
                    _a.sent();
                    return [3 /*break*/, 4];
                case 3:
                    e_17 = _a.sent();
                    expect(function () {
                        throw e_17;
                    }).not.toThrow();
                    return [3 /*break*/, 4];
                case 4:
                    auth.setTokens(null);
                    _a.label = 5;
                case 5:
                    _a.trys.push([5, 7, , 8]);
                    return [4 /*yield*/, axiosInstance.get('custom')];
                case 6:
                    _a.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 8];
                case 7:
                    e_18 = _a.sent();
                    expect(function () {
                        throw e_18;
                    }).toThrow(/Request failed with status code 401.*/);
                    return [3 /*break*/, 8];
                case 8: return [2 /*return*/];
            }
        });
    }); });
    it('fails to set token synchronously when persistentTokenStorage is async', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var axiosMock = new MockAdapter(axiosInstance);
        axiosMock.onGet('custom').reply(function (config) {
            if (config.headers.Authorization === 'Bearer accessTokenValue') {
                return [200];
            }
            else {
                return [401];
            }
        });
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        var persistentTokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(localStorage), false);
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            persistentTokenStorage: persistentTokenStorage,
            loadTokensFromStorage: false
        });
        expect(function () { return auth.setTokens({ access: { value: 'accessTokenValue' } }); }).toThrow('persistentTokenStorage is async. Use setTokensAsync method instead');
        expect(function () { return auth.setTokens(null); }).toThrow('persistentTokenStorage is async. Use setTokensAsync method instead');
    });
    it('fails to set token synchronously when tokenStorage is async', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var axiosMock = new MockAdapter(axiosInstance);
        axiosMock.onGet('custom').reply(function (config) {
            if (config.headers.Authorization === 'Bearer accessTokenValue') {
                return [200];
            }
            else {
                return [401];
            }
        });
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        var tokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(localStorage), false);
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            tokenStorage: tokenStorage,
            loadTokensFromStorage: false
        });
        expect(function () { return auth.setTokens({ access: { value: 'accessTokenValue' } }); }).toThrow('tokenStorage is async. Use setTokensAsync method instead');
        expect(function () { return auth.setTokens(null); }).toThrow('tokenStorage is async. Use setTokensAsync method instead');
    });
    it('fails to load token synchronously when persistentTokenStorage is async', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var axiosMock = new MockAdapter(axiosInstance);
        axiosMock.onGet('custom').reply(function (config) {
            if (config.headers.Authorization === 'Bearer accessTokenValue') {
                return [200];
            }
            else {
                return [401];
            }
        });
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        var persistentTokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(localStorage), false);
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
            persistentTokenStorage: persistentTokenStorage
        });
        expect(function () { return auth.loadTokensFromStorage(); }).toThrow('persistentTokenStorage is async. Use loadTokensFromStorageAsync method instead');
    });
    it('fails to load tokens synchronously when tokenStorage is async', function () {
        var axiosInstance = axios.create();
        var axiosAdapter = new AxiosAdapter(axiosInstance);
        var axiosMock = new MockAdapter(axiosInstance);
        axiosMock.onGet('custom').reply(function (config) {
            if (config.headers.Authorization === 'Bearer accessTokenValue') {
                return [200];
            }
            else {
                return [401];
            }
        });
        var openidConnectAdapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: { method: 'post', url: 'login' }
        };
        var tokenStorage = new TokenStorageAsyncAdapter(new DefaultTokenStorage(localStorage), false);
        var auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, { tokenStorage: tokenStorage });
        expect(function () { return auth.loadTokensFromStorage(); }).toThrow('tokenStorage is async. Use loadTokensFromStorageAsync method instead');
    });
    it('loads tokens asynchronously when tokenStorage and persistentTokenStorage are sync', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValue') {
                            return [200];
                        }
                        else {
                            return [401];
                        }
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter);
                    return [4 /*yield*/, auth.loadTokensFromStorageAsync()];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it('does not fail when loading tokens asynchronously when tokenStorage and persistentTokenStorage are null', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValue') {
                            return [200];
                        }
                        else {
                            return [401];
                        }
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: null,
                        tokenStorage: null,
                        persistentTokenStorage: null,
                        loadTokensFromStorage: false
                    });
                    return [4 /*yield*/, auth.loadTokensFromStorageAsync()];
                case 1:
                    _a.sent();
                    expect(auth.getTokens()).toBeUndefined();
                    return [2 /*return*/];
            }
        });
    }); });
    it('does not fail when loading tokens asynchronously when tokenStorage is sync and persistentTokenStorage is null', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosAdapter, axiosMock, openidConnectAdapter, serverConfiguration, auth;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    axiosMock = new MockAdapter(axiosInstance);
                    axiosMock.onGet('custom').reply(function (config) {
                        if (config.headers.Authorization === 'Bearer accessTokenValue') {
                            return [200];
                        }
                        else {
                            return [401];
                        }
                    });
                    openidConnectAdapter = new OpenidConnectAdapter();
                    serverConfiguration = {
                        loginEndpoint: { method: 'post', url: 'login' }
                    };
                    sessionStorage.setItem('auth.accessToken', 'accessTokenValue');
                    auth = new Auth(serverConfiguration, openidConnectAdapter, axiosAdapter, {
                        accessTokenDecoder: null,
                        persistentTokenStorage: null,
                        loadTokensFromStorage: false
                    });
                    return [4 /*yield*/, auth.loadTokensFromStorageAsync()];
                case 1:
                    _a.sent();
                    expect(function () { return auth.getTokens(); });
                    return [2 /*return*/];
            }
        });
    }); });
});
//# sourceMappingURL=auth.test.js.map