var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { AxiosAdapter, OpenidConnectAdapter, openidConnectDiscovery } from '../../src';
import axios from 'axios';
import { advanceTo, clear } from 'jest-date-mock';
import MockAdapter from 'axios-mock-adapter';
describe('Openid Connect Adapter - Discovery', function () {
    it('build configuration from /.well-known/openid-configuration', function () { return __awaiter(void 0, void 0, void 0, function () {
        var axiosInstance, axiosMock, axiosAdapter, configuration;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    axiosInstance = axios.create();
                    axiosMock = new MockAdapter(axiosInstance);
                    // Mock any GET request to /users
                    // arguments for reply are (status, data, headers)
                    axiosMock.onGet('issuer/.well-known/openid-configuration').reply(200, {
                        token_endpoint: 'tokenEndpoint',
                        end_session_endpoint: 'endSessionEndpoint'
                    });
                    axiosAdapter = new AxiosAdapter(axiosInstance);
                    return [4 /*yield*/, openidConnectDiscovery(axiosAdapter, 'issuer')];
                case 1:
                    configuration = _a.sent();
                    expect(configuration.loginEndpoint).toEqual({ method: 'POST', url: 'tokenEndpoint' });
                    expect(configuration.renewEndpoint).toEqual({ method: 'POST', url: 'tokenEndpoint' });
                    expect(configuration.logoutEndpoint).toEqual({ method: 'POST', url: 'endSessionEndpoint' });
                    return [2 /*return*/, configuration];
            }
        });
    }); });
});
describe('Openid Connect Adapter', function () {
    beforeEach(function () {
        advanceTo(133713371337);
    });
    afterEach(function () {
        clear();
    });
    it('default module is defined', function () {
        expect(OpenidConnectAdapter).toBeDefined();
    });
    it('adapts login request', function () {
        var adapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: {
                method: 'POST',
                url: 'url'
            }
        };
        var headers = { 'Content-Type': 'application/x-www-form-urlencoded' };
        var username = 'testUsername';
        var password = 'testPassword';
        var loginRequest = adapter.asLoginRequest(serverConfiguration, { username: username, password: password });
        expect(loginRequest.method).toBe(serverConfiguration.loginEndpoint.method);
        expect(loginRequest.url).toBe(serverConfiguration.loginEndpoint.url);
        expect(loginRequest.headers).toEqual(headers);
        expect(loginRequest.data).toBe("grant_type=password&username=" + username + "&password=" + password);
    });
    it('adapts logout request', function () {
        var adapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: {
                method: 'POST',
                url: 'loginUrl'
            },
            logoutEndpoint: {
                method: 'POST',
                url: 'logoutUrl'
            }
        };
        var headers = { 'Content-Type': 'application/x-www-form-urlencoded' };
        var tokens = {
            access: {
                value: 'testAccessToken'
            },
            refresh: {
                value: 'testRefreshToken'
            }
        };
        var logoutRequest = adapter.asLogoutRequest(serverConfiguration, tokens);
        expect(logoutRequest).not.toBeFalsy();
        if (logoutRequest != null) {
            expect(logoutRequest.method).toBe(serverConfiguration.logoutEndpoint.method);
            expect(logoutRequest.url).toBe(serverConfiguration.logoutEndpoint.url);
            expect(logoutRequest.headers).toEqual(headers);
            expect(logoutRequest.data).toBe("refresh_token=" + tokens.refresh.value);
        }
    });
    it('adapts renew request', function () {
        var adapter = new OpenidConnectAdapter();
        var serverConfiguration = {
            loginEndpoint: {
                method: 'POST',
                url: 'loginUrl'
            }
        };
        var headers = { 'Content-Type': 'application/x-www-form-urlencoded' };
        var tokens = {
            access: {
                value: 'testAccessToken'
            },
            refresh: {
                value: 'testRefreshToken'
            }
        };
        var renewRequest = adapter.asRenewRequest(serverConfiguration, tokens);
        if (renewRequest != null) {
            expect(renewRequest.method).toBe(serverConfiguration.renewEndpoint.method);
            expect(renewRequest.url).toBe(serverConfiguration.renewEndpoint.url);
            expect(renewRequest.headers).toEqual(headers);
            expect(renewRequest.data).toBe("grant_type=refresh_token&refresh_token=" + tokens.refresh.value);
        }
    });
    it('retrieves tokens from response without expires', function () {
        var adapter = new OpenidConnectAdapter();
        var response = {
            data: {
                access_token: 'accessToken',
                refresh_token: 'refreshToken'
            }
        };
        var tokens = adapter.getResponseTokens(response);
        expect(tokens.access).toEqual({ value: response.data.access_token });
        expect(tokens.refresh).toBeDefined();
        if (tokens.refresh != null) {
            expect(tokens.refresh).toEqual({ value: response.data.refresh_token });
        }
    });
    it('retrieves tokens from response with expires', function () {
        var adapter = new OpenidConnectAdapter();
        var response = {
            data: {
                access_token: 'accessToken',
                refresh_token: 'refreshToken',
                expires_in: 30,
                refresh_expires_in: 45
            }
        };
        var tokens = adapter.getResponseTokens(response);
        var expiresAt = new Date();
        var refreshExpiresAt = new Date();
        expiresAt.setSeconds(expiresAt.getSeconds() + response.data.expires_in);
        refreshExpiresAt.setSeconds(refreshExpiresAt.getSeconds() + response.data.refresh_expires_in);
        expect(tokens.access).toEqual({ value: response.data.access_token, expiresAt: expiresAt });
        expect(tokens.refresh).toBeDefined();
        if (tokens.refresh != null) {
            expect(tokens.refresh).toEqual({
                value: response.data.refresh_token,
                expiresAt: refreshExpiresAt
            });
        }
    });
    it('set access token on request', function () {
        var adapter = new OpenidConnectAdapter();
        var request = { method: 'GET', url: 'url' };
        var tokens = {
            access: {
                value: 'testAccessToken'
            }
        };
        adapter.configureRequest(request, tokens);
        expect(request.headers).toEqual({ Authorization: "Bearer " + tokens.access.value });
    });
    it('set access token on request with custom headers', function () {
        var adapter = new OpenidConnectAdapter();
        var request = { method: 'GET', url: 'url', headers: { dummy: 'testDummy' } };
        var tokens = {
            access: {
                value: 'testAccessToken'
            }
        };
        adapter.configureRequest(request, tokens);
        expect(request.headers).toEqual({
            Authorization: "Bearer " + tokens.access.value,
            dummy: 'testDummy'
        });
    });
    it('accessTokenHasExpired returns false when authorization header is not present in request', function () {
        var adapter = new OpenidConnectAdapter();
        var request = { method: 'GET', url: 'url' };
        var response = { status: 401, data: { error: 'invalid_token' } };
        var expired = adapter.shouldRenew(request, response);
        expect(expired).toBeFalsy();
    });
    it('accessTokenHasExpired returns true when authorization header is present in request', function () {
        var adapter = new OpenidConnectAdapter();
        var request = {
            method: 'GET',
            url: 'url',
            headers: { Authorization: 'Bearer accessToken' }
        };
        var response = { status: 401, data: { error: 'invalid_token' } };
        var expired = adapter.shouldRenew(request, response);
        expect(expired).toBeTruthy();
    });
    it('refreshTokenHasExpired returns false when authorization header is not present in request', function () {
        var adapter = new OpenidConnectAdapter();
        var request = { method: 'GET', url: 'url' };
        var response = { status: 400, data: { error: 'invalid_grant' } };
        var expired = adapter.isExpired(request, response);
        expect(expired).toBeFalsy();
    });
    it('refreshTokenHasExpired returns true when authorization header is present in request', function () {
        var adapter = new OpenidConnectAdapter();
        var request = {
            method: 'GET',
            url: 'url',
            headers: { Authorization: 'Bearer accessToken' }
        };
        var response = { status: 400, data: { error: 'invalid_grant' } };
        var expired = adapter.isExpired(request, response);
        expect(expired).toBeTruthy();
    });
});
//# sourceMappingURL=openid-connect-adapter.test.js.map