import { DefaultTokenDecoder } from '../../src';
import { advanceTo, clear } from 'jest-date-mock';
describe('Default Token Decoder', function () {
    beforeEach(function () {
        advanceTo(133713371337);
    });
    afterEach(function () {
        clear();
    });
    it('default module is defined', function () {
        expect(DefaultTokenDecoder).toBeDefined();
    });
    it('token is expired', function () {
        var date = new Date();
        var token = { value: 'accessTokenValue', expiresAt: date };
        var tokenDecoder = new DefaultTokenDecoder();
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeTruthy();
    });
    it('token is expired when 1 second after expiration', function () {
        var date = new Date();
        date.setSeconds(date.getSeconds() - 1);
        var token = { value: 'accessTokenValue', expiresAt: date };
        var tokenDecoder = new DefaultTokenDecoder();
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeTruthy();
    });
    it('token is not expired when 1 second before expiration', function () {
        var date = new Date();
        date.setSeconds(date.getSeconds() + 1);
        var token = { value: 'accessTokenValue', expiresAt: date };
        var tokenDecoder = new DefaultTokenDecoder();
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeFalsy();
    });
    it('token is not expired with offset', function () {
        var date = new Date();
        date.setSeconds(date.getSeconds() - 9);
        var token = { value: 'accessTokenValue', expiresAt: date };
        var tokenDecoder = new DefaultTokenDecoder(10);
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeFalsy();
    });
    it('token is not expired when expiresAt is not defined', function () {
        var token = { value: 'accessTokenValue' };
        var tokenDecoder = new DefaultTokenDecoder();
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeFalsy();
    });
});
//# sourceMappingURL=default-token-decoder.test.js.map