import { JwtTokenDecoder } from '../../src';
import { advanceTo, clear } from 'jest-date-mock';
describe('Jwt Token Decoder', function () {
    var accessToken = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM3MSwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.uXiL5Yu-Ip0iNkvmK54U5MHDEhE0M6KsNFAb-BWg6oQ';
    var accessTokenExpNow = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM3MSwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.uXiL5Yu-Ip0iNkvmK54U5MHDEhE0M6KsNFAb-BWg6oQ';
    var accessTokenExpNowMinus1 = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM3MCwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.5qqM6z2xTN_DqFCJYJeRxFlbMKJ0AyIrudGqvHCke3g';
    var accessTokenExpNowPlus1 = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM3MiwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.LiBxgT3VwrIp7oOfBWbEcKRFIQXqX8idxdhdA3N8D7Q';
    var accessTokenExpNowMinus10 = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM2MSwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.FTB2JxqDgVwZH7WPEKsENtbXZ3l01w-bbZJQqfY0xc8';
    var accessTokenExpNowMinus9 = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJKd3QgVG9vbGJveCIsImlhdCI6MTMzNzEzMzExLCJleHAiOjEzMzcxMzM2MiwiYXVkIjoiand0LXRvb2xib3giLCJzdWIiOiJqd3QtdG9vbGJveCJ9.feOjEJUK5lS80e9tysEty6c3E15A9xyNl5RcFeBzz5k';
    beforeEach(function () {
        advanceTo(133713371337);
    });
    afterEach(function () {
        clear();
    });
    it('default module is defined', function () {
        expect(JwtTokenDecoder).toBeDefined();
    });
    it('decode token', function () {
        var token = { value: accessToken };
        var tokenDecoder = new JwtTokenDecoder();
        var decoded = tokenDecoder.decode(token);
        expect(decoded).toEqual({
            iss: 'Jwt Toolbox',
            iat: 133713311,
            exp: 133713371,
            aud: 'jwt-toolbox',
            sub: 'jwt-toolbox'
        });
    });
    it('check token is expired', function () {
        var token = { value: accessTokenExpNow };
        var tokenDecoder = new JwtTokenDecoder();
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeTruthy();
    });
    it('check token is expired when 1 second after expiration', function () {
        var token = { value: accessTokenExpNowMinus1 };
        var tokenDecoder = new JwtTokenDecoder();
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeTruthy();
    });
    it('check token is not expired when 1 second before expiration', function () {
        var token = { value: accessTokenExpNowPlus1 };
        var tokenDecoder = new JwtTokenDecoder();
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeFalsy();
    });
    it('check token is not expired with offset', function () {
        var token = { value: accessTokenExpNowMinus9 };
        var tokenDecoder = new JwtTokenDecoder(10);
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeFalsy();
    });
    it('check token is expired with offset', function () {
        var token = { value: accessTokenExpNowMinus10 };
        var tokenDecoder = new JwtTokenDecoder(10);
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeTruthy();
    });
    it('check token not is expired with NaN offset', function () {
        var token = { value: accessTokenExpNowMinus1 };
        var tokenDecoder = new JwtTokenDecoder(NaN);
        var accessTokenExpired = tokenDecoder.isExpired(token);
        expect(accessTokenExpired).toBeFalsy();
    });
});
//# sourceMappingURL=jwt-token-decoder.test.js.map